math.randomseed(os.time())

-- random waves
function make_waves(n)
	n = n or 7
	waves = {}
	phi = 0
	for i = 1,n do
		phi = phi + math.pi / n
		local x,y = math.random() - .5, math.random() - .5
		waves[i] = {x,y, phi, math.random() * i / n + n * 1.5}
	end
end

---- quasicrystal waves
--function make_waves()
--	waves = {}
--	for phi = 0,math.pi,math.pi/7 do
--		waves[#waves+1] = {0,0, phi, 15}
--	end
--end

function love.load()
	img = love.image.newImageData(1,1)
	img:setPixel(0,0, 255,255,255,255)
	img = love.graphics.newImage(img)

	make_waves()

	effect = [[
	#define TWOPI 2 * 3.1415926535898
	extern number phase;
	extern number rot;
	extern vec4[$N$] waves;

	number _hue(number s, number t, number h)
	{
		h = mod(h, 1.);
		number six_h = 6.0 * h;
		if (six_h < 1.) return (t-s) * six_h + s;
		if (six_h < 3.) return t;
		if (six_h < 4.) return (t-s) * (4.-six_h) + s;
		return s;
	}

	vec4 hsl_to_rgb(vec4 c)
	{
		if (c.y == 0)
			return vec4(vec3(c.z), c.a);

		number t = (c.z < .5) ? c.y*c.z + c.z : -c.y*c.z + (c.y+c.z);
		number s = 2.0 * c.z - t;
		#define Q 1.0/3.0
		return vec4(_hue(s,t,c.x+Q), _hue(s,t,c.x), _hue(s,t,c.x-Q), c.w);
	}

	number wrap(number x)
	{
		//return sin(x + phase) / 2. + .5;
		// triangle function
		number y = (x - floor(x + .5));
		if (mod(floor(x - .5), 2.0) != 0.0)
			y *= -1;
		return y + .5;
	}

	number wave(vec2 p, number phi, number f)
	{
		return sin((sin(phi+phase*rot)*p.x + cos(phi+phase*rot)*p.y) * TWOPI * f - phase);
	}

	vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
	{
		tc.x -= .5;
		tc.y -= .5;
		number amp = .0;
		$loop$
		return hsl_to_rgb(
			vec4(wrap(amp)*.1 + phase/10., wrap(amp)*.5 + .5, wrap(amp), 1.)
		);
	}
	]]
	local wave_loop = {}
	for i = 1,#waves do
		wave_loop[i] = ("amp += wave(tc+waves[%d].xy, waves[%d].z, waves[%d].w);"):format(i-1, i-1, i-1)
	end

	effect = effect:gsub("%b$$", { ["$N$"] =  #waves, ["$loop$"] = table.concat(wave_loop) })
	--local i = 0
	--for line in effect:gmatch('[^\n]*\n') do
	--	i = i + 1
	--	print(i, line:sub(1,-2))
	--end
	effect = love.graphics.newPixelEffect(effect)

	effect:send('phase', 0)
	effect:send('rot', 0)
	rot = false
	effect:send('waves', unpack(waves))
end

local t = 0
local trot = 0
function love.update(dt)
	local x,y = love.mouse.getPosition()
	for i = 1,#waves do
		waves[1][1] = x / love.graphics.getWidth() - .5
		waves[1][2] = y / love.graphics.getHeight() - .5
	end

	t = t + dt
	effect:send('phase', t)
	if rot then
		trot = trot + dt
		effect:send('rot', math.sin(trot/20) / 10)
	end
	effect:send('waves', unpack(waves))
end

function love.draw()
	love.graphics.setPixelEffect(effect)
	love.graphics.setColor(237,86,7)
	love.graphics.draw(img, 0,0, 0, love.graphics.getWidth(),love.graphics.getHeight())
end

function love.keypressed(key)
	if key == '+' or key == 'up' or key == 'w' then
		for i = 1,#waves do
			waves[i][4] = waves[i][4] / 1.1
		end
	elseif key == '-' or key == 'down' or key == 's' then
		for i = 1,#waves do
			waves[i][4] = waves[i][4] * 1.1
		end
	elseif key == ' ' then
		make_waves()
	elseif key == 'r' then
		rot = not rot
	elseif key == 'q' then
		love.event.push('q')
	end
end


function love.mousepressed(x,y,btn)
	if btn == 'wu' then
		love.keypressed('+')
	elseif btn == 'wd' then
		love.keypressed('-')
	elseif btn == 'l' then
		love.keypressed(' ')
	elseif btn == 'r' then
		love.keypressed('r')
	end
end
